# -*- coding:utf-8 -*-

import os
import ConfigParser
import platform
import logging
import threading

from multiprocessing import cpu_count

DEFAULT_LISTEN_PORT         = 2121
DEFAULT_MAX_CONNECTION      = 1024
DEFAULT_PARALLEL_UPLOAD_NUM = cpu_count() * 4
DEFAULT_MULTIPART_SIZE      = 10485760
DEFAULT_APPEND_BLOCK_SIZE   = 10485760

logger = logging.getLogger(__name__)

def get_str_value(cfg, section, option, default_value):
    """ get value from config
    :returns: str of section 
    """
    if cfg.has_section(section) and cfg.has_option(section, option):
        return cfg.get(section, option)
    return default_value 
def get_int_value(cfg, section, option, default_value):
    if cfg.has_section(section) and cfg.has_option(section, option): 
        return int(cfg.get(section, option))
    return default_value

class BosftpCfg(object):
    _instance_lock = threading.Lock()
    __init_already = False
    @classmethod
    def __new__(cls, *args, **kwargs):
        if not hasattr(BosftpCfg, "_instance"):
            with BosftpCfg._instance_lock:
                if not hasattr(BosftpCfg, "_instance"):
                    BosftpCfg._instance = object.__new__(cls)
        return BosftpCfg._instance

    @classmethod
    def _check_ipv4(cls, ipv4):
        if len(ipv4.split(".")) != 4:
            return False
        for value in ipv4.split("."):
            try:
                value = int(value)
                if value < 0 or value > 255:
                    return False
            except ValueError:
                return False

        return True

    def __init__(self):
        if not self.__init_already:
            cfg = ConfigParser.RawConfigParser()
            conf_path = os.path.dirname(
                os.path.dirname(
                    os.path.dirname(os.path.realpath(__file__)))) + "/bosftp/conf/bosftp.conf"
            cfg.read(conf_path)

            # ACCOUNT
            self.ak = cfg.get("BOS_ACCOUNT", "access_key_id")
            self.sk= cfg.get("BOS_ACCOUNT", "secret_access_key")
            self.bucket = cfg.get("BOS_ACCOUNT", "bucket") 
            self.homedir = cfg.get("BOS_ACCOUNT", "bos_user_home_dir")
            if not os.path.exists(self.homedir):
                os.mkdir(self.homedir)

            login_users = cfg.get("FTP_ACCOUNT", "login_users")
            login_users = login_users.strip(" ").split(";")
            self.login_users = list()
            # self.login_users 的结构为 [ (user1,pass1,RW), (user2,pass2,RW) ]
            for element in login_users:
                login_user = element.split(":")
                self.login_users.append(tuple(login_user))
            # NETWORK
            self.host = get_str_value(cfg, "NETWORK", "endpoint", "http://bj.bcebos.com")
            assert self.host != "" 
            self.connection_timeout_in_mills = get_int_value(
                cfg, "NETWORK", "connection_timeout_in_mills", -1) 
            self.listen_port = get_int_value(cfg, "NETWORK", "listen_port", DEFAULT_LISTEN_PORT)

            # masquerade_address
            self.masquerade_address = None
            if cfg.has_section("NETWORK") and cfg.has_option("NETWORK", "masquerade_address") and str(
                    cfg.get("NETWORK", "masquerade_address")) != "":
                if BosftpCfg._check_ipv4(cfg.get("NETWORK", "masquerade_address")):
                    self.masquerade_address = cfg.get("NETWORK", "masquerade_address")

            passive_ports = cfg.get("NETWORK", 'passive_port').split(',')
            if len(passive_ports) > 1:
                self.passive_ports = range(int(passive_ports[0]), int(passive_ports[1]))
            elif len(passive_ports) == 1:
                self.passive_ports = range(int(passive_ports[0]), 65535)
            else:
                self.passive_ports = range(60000, 65535)

            #assert self.listen_port > 60000 and self.listen_port < 65535 
            self.parallel_upload_num = get_int_value(cfg, "NETWORK", "parallel_upload_num", cpu_count() * 4) 
            assert self.parallel_upload_num > 0 and self.parallel_upload_num < 1000
            self.max_connection_num = get_int_value(
                cfg, "NETWORK", "max_connection_num", DEFAULT_MAX_CONNECTION) 
            assert self.max_connection_num > 0
            self.multi_part_size = get_int_value(
                cfg, "NETWORK", "multi_part_size", DEFAULT_MULTIPART_SIZE)
            # 5m <= multi_part_size <= 5G
            assert self.multi_part_size >= 5242880 and self.multi_part_size <= 5368709120  
            self.appe_block_size = get_int_value(cfg, "NETWORK", "appe_block_size", DEFAULT_APPEND_BLOCK_SIZE)
            # LOG
            self.log_level = logging.INFO
            if cfg.has_section("LOG") and cfg.has_option("LOG", "log_level"):
                if str(cfg.get("LOG", "log_level")).lower() == str("WARNING").lower():
                    self.log_level = logging.WARN
                if str(cfg.get("LOG", "log_level")).lower() == str("INFO").lower():
                    self.log_level = logging.INFO 
                if str(cfg.get("LOG", "log_level")).lower() == str("DEBUG").lower(): 
                    self.log_level = logging.DEBUG 
                if str(cfg.get("LOG", "log_level")).lower() == str("ERROR").lower(): 
                    self.log_level = logging.ERROR 
                if str(cfg.get("LOG", "log_level")).lower() == str("CRITICAL").lower():
                    self.log_level = logging.CRITICAL

            self.log_dir = "log"
            if cfg.has_section("LOG") and cfg.has_option("LOG", "log_dir"):
                self.log_dir = str(cfg.get("LOG", "log_dir")).lower()
            if not os.path.exists(self.log_dir):
                os.mkdir(self.log_dir)

            self.log_file_name = "bosftp.log"
            if str(self.log_dir).endswith("/"):
                self.log_file_name = self.log_dir + self.log_file_name
            else:
                self.log_file_name = self.log_dir + "/" + self.log_file_name

            self.__class__.__init_already = True

    def __str__(self):
        return "access_key: %s \n" \
               "secret_key: %s \n" \
               "bucket: %s \n" \
               "homedir: %s \n" \
               "login_users: %s \n" \
               "host: %s \n" \
               "listen_port: %d \n" \
               "masquerade_address: %s \n" \
               "connection_timeout_in_mills: %s \n" \
               "parallel_upload_num:%d \n" \
               "max_connection_num: %d \n" \
               "log_level: %d \n" \
               "log_dir: %s \n" \
               "log_file_name: %s" % (
                   self.ak, self.sk, self.bucket, self.homedir,
                   self.login_users, self.host, self.listen_port, self.masquerade_address, self.connection_timeout_in_mills,
                   self.parallel_upload_num, self.max_connection_num, self.log_level, self.log_dir, self.log_file_name)


# for unit test
if __name__ == "__main__":
    print BosftpCfg().ak
    print BosftpCfg().ak
